// quickload.c
// Simple program to create MAME(tm)-compatible NVRAM files for CPS-3 games, from the files located on the game CD
// Saves waiting to Capcom's slow-ass loader to do it normally :)
// Written by Barry Rodewald
// 4 July 2007

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VER "1.01"

void load_simm(FILE*, unsigned char*, int);

int main (int argc, char** argv)
{
	FILE* infile;
	FILE* outfile;
	char infilepath[250];
	char infilename[250];
	char outfilepath[250];
	unsigned char* src;
	unsigned char* dest;
	int sampleswap = 0;
	int x,y;

	printf("CPS-3 QUICKLOAD %s\n\n",VER);
	if(argc != 3)
	{
		printf("Usage:  quickload <path to extracted CD files> <output NVRAM file>\n\n");
		exit(-1);
	}
	
	strcpy(infilepath,argv[1]);
	strcpy(outfilepath,argv[2]);

	x = strlen(infilepath);
	if(infilepath[x - 1] != '/' && infilepath[x - 1] != '\\')
	{
		infilepath[x] = '/';  // add slash to pathname if one isn't already there
		infilepath[x+1] = '\0';
	}

	printf("Input path: %s\n",infilepath);
	printf("Output: %s\n",outfilepath);

	// open output file
	outfile = fopen(outfilepath,"wb");

	if(outfile == NULL)
	{
		printf("Error:  File '%s' cannout be opened for writing\n",outfilepath);
		exit(-2);
	}

	// write fake EEPROM data (all FF)
	printf("Writing blank EEPROM data...\n");
	dest = malloc(0x400);
	memset(dest,0xff,0x400);
	fwrite(dest,1,0x400,outfile);
	free(dest);

	// write SH-2 program code (SIMM1 and SIMM2)
	dest = malloc(0x800000);

	printf("Loading SIMM1...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"10");  // SIMM1
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);

	{
		unsigned char val;
		int ret;
		for(x=0;x<0x200000;x++)
		{
			ret = fread(&val,1,1,infile);
			if(ret != 0)
				dest[x] = val;
			else
				dest[x] = 0xff;
			ret = fread(&val,1,1,infile);
			if(ret != 0)
				dest[x + 0x200000] = val;
			else
				dest[x + 0x200000] = 0xff;
			ret = fread(&val,1,1,infile);
			if(ret != 0)
				dest[x + 0x400000] = val;
			else
				dest[x + 0x400000] = 0xff;
			ret = fread(&val,1,1,infile);
			if(ret != 0)
				dest[x + 0x600000] = val;
			else
				dest[x + 0x600000] = 0xff;
		}
	}
	fclose(infile);
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	printf("Loading SIMM2...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"20");  // SIMM1
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);

	if(infile != NULL)
	{
		{
			unsigned char val;
			int ret;
			for(x=0;x<0x200000;x++)
			{
				ret = fread(&val,1,1,infile);
				if(ret != 0)
					dest[x] = val;
				else
					dest[x] = 0xff;
				ret = fread(&val,1,1,infile);
				if(ret != 0)
					dest[x + 0x200000] = val;
				else
					dest[x + 0x200000] = 0xff;
				ret = fread(&val,1,1,infile);
				if(ret != 0)
					dest[x + 0x400000] = val;
				else
					dest[x + 0x400000] = 0xff;
				ret = fread(&val,1,1,infile);
				if(ret != 0)
					dest[x + 0x600000] = val;
				else
					dest[x + 0x600000] = 0xff;
			}
		}
	fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	// load sound samples
	printf("Loading SIMM3, side 1...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"30");  // SIMM3
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);


	printf("Loading SIMM3, side 2...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"31");  // SIMM3
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	// other data
	printf("Loading SIMM4, side 1...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"40");  // SIMM4
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);


	printf("Loading SIMM4, side 2...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"41");  // SIMM4
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	printf("Loading SIMM5, side 1...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"50");  // SIMM5
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);


	printf("Loading SIMM5, side 2...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"51");  // SIMM5
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	printf("Loading SIMM6, side 1...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"60");  // SIMM6
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);


	printf("Loading SIMM6, side 2...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"61");  // SIMM6
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	printf("Loading SIMM7, side 1...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"70");  // SIMM7
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	printf("Loading SIMM7, side 2...\n");
	strcpy(infilename,infilepath);
	strcat(infilename,"71");  // SIMM7
	infile = fopen(infilename,"rb");
	memset(dest,0xff,0x800000);
	
	if(infile != NULL)
	{
		load_simm(infile,dest,0);
		fclose(infile);
	}
	printf("Writing to NVRAM file...\n");
	fwrite(dest,1,0x800000,outfile);

	free(dest);

	// clean up
	fclose(outfile);

	printf("\nFinished.\n\n");
}

void load_simm(FILE* f,unsigned char* dest,int swap)
{
	int x;
	int notswap;
	int ret;
	unsigned char val;

	for(x=0;x<0x200000;x++)
	{
		ret = fread(&val,1,1,f);
		if(ret != 0)
			dest[x] = val;
		else
			dest[x] = 0xff;
		ret = fread(&val,1,1,f);
		if(ret != 0)
			dest[x + 0x200000] = val;
		else
			dest[x + 0x200000] = 0xff;
	}
	for(x=0;x<0x200000;x++)
	{
		ret = fread(&val,1,1,f);
		if(ret != 0)
			dest[x + 0x400000] = val;
		else
			dest[x + 0x400000] = 0xff;
		ret = fread(&val,1,1,f);
		if(ret != 0)
			dest[x + 0x600000] = val;
		else
			dest[x + 0x600000] = 0xff;
	}
	if(swap == 1)  // byteswap
	{
		unsigned char temp;
		
		printf("Byteswapping...");
		for(x=0;x<0x800000;x+=2)
		{
			temp = dest[x];
			dest[x] = dest[x+1];
			dest[x+1] = temp;
		}
	}
}
